package com.example

import org.springframework.boot.autoconfigure.SpringBootApplication
import org.springframework.boot.runApplication
import org.springframework.context.annotation.Bean
import org.springframework.web.reactive.function.server.*
import reactor.core.publisher.Mono

suspend fun coroutineHandler(req: ServerRequest): ServerResponse {
    val principal = req.awaitPrincipal()
    return ServerResponse.ok().bodyAndAwait(principal!!)
}

fun classicHandler(req: ServerRequest): Mono<ServerResponse> =
        req.principal()
                .flatMap { ServerResponse.ok().syncBody(it) }

@SpringBootApplication
open class Application {

    @Bean
    open fun router(): RouterFunction<ServerResponse> {
        val co = coRouter {
            GET("/co", ::coroutineHandler)
        }
        val classic = router {
            GET("/classic", ::classicHandler)
        }
        return co.and(classic)
    }

}

fun main(args: Array<String>) {
    runApplication<Application>(*args)
}